# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.lookup_tables import (
    lookup_table_revision_data_copy_params,
    lookup_table_revision_data_update_params,
    lookup_table_revision_data_archieve_params,
    lookup_table_revision_data_retrieve_params,
    lookup_table_revision_data_delete_key_params,
    lookup_table_revision_data_update_key_params,
    lookup_table_revision_data_generate_download_url_params,
)
from .lookup_table_revision_data_jobs import (
    LookupTableRevisionDataJobsResource,
    AsyncLookupTableRevisionDataJobsResource,
    LookupTableRevisionDataJobsResourceWithRawResponse,
    AsyncLookupTableRevisionDataJobsResourceWithRawResponse,
    LookupTableRevisionDataJobsResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse,
)
from ....types.lookup_tables.lookup_table_revision_data_copy_response import LookupTableRevisionDataCopyResponse
from ....types.lookup_tables.lookup_table_revision_data_delete_response import LookupTableRevisionDataDeleteResponse
from ....types.lookup_tables.lookup_table_revision_data_update_response import LookupTableRevisionDataUpdateResponse
from ....types.lookup_tables.lookup_table_revision_data_archieve_response import LookupTableRevisionDataArchieveResponse
from ....types.lookup_tables.lookup_table_revision_data_retrieve_response import LookupTableRevisionDataRetrieveResponse
from ....types.lookup_tables.lookup_table_revision_data_delete_key_response import (
    LookupTableRevisionDataDeleteKeyResponse,
)
from ....types.lookup_tables.lookup_table_revision_data_update_key_response import (
    LookupTableRevisionDataUpdateKeyResponse,
)
from ....types.lookup_tables.lookup_table_revision_data_retrieve_key_response import (
    LookupTableRevisionDataRetrieveKeyResponse,
)
from ....types.lookup_tables.lookup_table_revision_data_generate_download_url_response import (
    LookupTableRevisionDataGenerateDownloadURLResponse,
)

__all__ = ["LookupTableRevisionDataResource", "AsyncLookupTableRevisionDataResource"]


class LookupTableRevisionDataResource(SyncAPIResource):
    @cached_property
    def lookup_table_revision_data_jobs(self) -> LookupTableRevisionDataJobsResource:
        return LookupTableRevisionDataJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> LookupTableRevisionDataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LookupTableRevisionDataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LookupTableRevisionDataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LookupTableRevisionDataResourceWithStreamingResponse(self)

    def retrieve(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        additional: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataRetrieveResponse:
        """
        List Lookup Table Revision Data items for the given UUID.

        Args:
          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for each Data item. You
              can then use a lookup key for the Get/Upsert/Delete data entry endpoints in this
              section.

          limit: The maximum number of Data items to return. Defaults to 2000. You can set this
              to return fewer items if required.

              If you expect the Revision to contain more than 2000 Data items, you can use the
              [Trigger Downlad URL Job](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/TriggerDownloadJob)
              to download the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional": additional,
                        "limit": limit,
                    },
                    lookup_table_revision_data_retrieve_params.LookupTableRevisionDataRetrieveParams,
                ),
            ),
            cast_to=LookupTableRevisionDataRetrieveResponse,
        )

    def update(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        items: Iterable[Dict[str, object]],
        additional: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataUpdateResponse:
        """
        Create/update the Lookup Table Revision Data for the given UUID.

        Args:
          items: The data for a lookup table revision

          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for each Data item. You
              can then use a lookup key for the Get/Upsert/Delete data entry endpoints in this
              section.

          version: The version of the LookupTableRevisionData.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            body=maybe_transform(
                {
                    "items": items,
                    "version": version,
                },
                lookup_table_revision_data_update_params.LookupTableRevisionDataUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"additional": additional},
                    lookup_table_revision_data_update_params.LookupTableRevisionDataUpdateParams,
                ),
            ),
            cast_to=LookupTableRevisionDataUpdateResponse,
        )

    def delete(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataDeleteResponse:
        """
        Delete the Lookup Table Revision Data for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataDeleteResponse,
        )

    def archieve(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_type: Literal["application/jsonl", "text/csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataArchieveResponse:
        """
        Get a URL which you can use to download the data for the specified archived
        Lookup Table Revision:

        - The `contentType` request parameter is required.
        - The returned URL is presigned - you can copy it into a browser and the data
          file is downloaded locally.
        - The upload URL is time limited - the `expiry` time is given in the response
          and the URL is valid for **_one hour_**.

        Args:
          content_type: The content type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/archived",
            body=maybe_transform(
                {"content_type": content_type},
                lookup_table_revision_data_archieve_params.LookupTableRevisionDataArchieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataArchieveResponse,
        )

    def copy(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        revision_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataCopyResponse:
        """
        Copy the Lookup Table Revision Data from a source Revision to an optional target
        Revision:

        - If you omit a target `revisionId`, then the source Revision and its Data is
          duplicated. The new Revision is given the source Revision's name appended with
          "Copy" but is assigned a new unique id.
        - If you specify a target `revisionId` to copy the source Revision and its Data
          to, you must ensure that the target Revision has a Data schema that matches
          the source Revision's Data schema otherwise you'll receive an error

        Args:
          revision_id: The target Revision id that the source Revision's data will be copied to.
              _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/copy",
            body=maybe_transform(
                {"revision_id": revision_id}, lookup_table_revision_data_copy_params.LookupTableRevisionDataCopyParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataCopyResponse,
        )

    def delete_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataDeleteKeyResponse:
        """
        Delete a Lookup Table Revision Data entry by lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          version: The version of the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            body=maybe_transform(
                {"version": version},
                lookup_table_revision_data_delete_key_params.LookupTableRevisionDataDeleteKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataDeleteKeyResponse,
        )

    def generate_download_url(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_length: int,
        content_type: Literal["application/jsonl", "text/csv"],
        file_name: str,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataGenerateDownloadURLResponse:
        """
        Generate a URL which can be used to upload a data file for creating or updating
        the Lookup Table Revision's data:

        - An upload URL is returned together with an UPLOAD `jobId`.
        - You can then upload your data file using a PUT request using the returned
          upload URL as the endpoint. For the PUT request, map the headers returned and
          their values and in the request body select the specified CSV or JSONL file
          containing the Revision Data to upload.
        - You can use the returned UPLOAD `jobId` with the
          [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
          or the
          [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
          endpoints for any follow-up or troubleshooting.

        **Important:**

        - The `contentLength` request parameter is required.
        - The upload URL is time limited - it is valid for **_one minute_**.

        Args:
          content_length: The size of the file body in bytes. For example: `"contentLength": 485`, where
              485 is the size in bytes of the file to upload.

          content_type: The content type

          file_name: The name of the file to be uploaded.

          version: Version of the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/generateuploadurl",
            body=maybe_transform(
                {
                    "content_length": content_length,
                    "content_type": content_type,
                    "file_name": file_name,
                    "version": version,
                },
                lookup_table_revision_data_generate_download_url_params.LookupTableRevisionDataGenerateDownloadURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataGenerateDownloadURLResponse,
        )

    def retrieve_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataRetrieveKeyResponse:
        """
        Retrieve a Lookup Table Revision Data item for the given lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataRetrieveKeyResponse,
        )

    def update_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        item: Dict[str, object],
        additional: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataUpdateKeyResponse:
        """
        Create/update a Lookup Table Revision Data item by lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          item: The item you want to upsert

          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for the Data item.

          version: The version of the LookupTableRevisionData.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            body=maybe_transform(
                {
                    "item": item,
                    "version": version,
                },
                lookup_table_revision_data_update_key_params.LookupTableRevisionDataUpdateKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"additional": additional},
                    lookup_table_revision_data_update_key_params.LookupTableRevisionDataUpdateKeyParams,
                ),
            ),
            cast_to=LookupTableRevisionDataUpdateKeyResponse,
        )


class AsyncLookupTableRevisionDataResource(AsyncAPIResource):
    @cached_property
    def lookup_table_revision_data_jobs(self) -> AsyncLookupTableRevisionDataJobsResource:
        return AsyncLookupTableRevisionDataJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLookupTableRevisionDataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLookupTableRevisionDataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLookupTableRevisionDataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLookupTableRevisionDataResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        additional: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataRetrieveResponse:
        """
        List Lookup Table Revision Data items for the given UUID.

        Args:
          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for each Data item. You
              can then use a lookup key for the Get/Upsert/Delete data entry endpoints in this
              section.

          limit: The maximum number of Data items to return. Defaults to 2000. You can set this
              to return fewer items if required.

              If you expect the Revision to contain more than 2000 Data items, you can use the
              [Trigger Downlad URL Job](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/TriggerDownloadJob)
              to download the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "additional": additional,
                        "limit": limit,
                    },
                    lookup_table_revision_data_retrieve_params.LookupTableRevisionDataRetrieveParams,
                ),
            ),
            cast_to=LookupTableRevisionDataRetrieveResponse,
        )

    async def update(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        items: Iterable[Dict[str, object]],
        additional: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataUpdateResponse:
        """
        Create/update the Lookup Table Revision Data for the given UUID.

        Args:
          items: The data for a lookup table revision

          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for each Data item. You
              can then use a lookup key for the Get/Upsert/Delete data entry endpoints in this
              section.

          version: The version of the LookupTableRevisionData.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            body=await async_maybe_transform(
                {
                    "items": items,
                    "version": version,
                },
                lookup_table_revision_data_update_params.LookupTableRevisionDataUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"additional": additional},
                    lookup_table_revision_data_update_params.LookupTableRevisionDataUpdateParams,
                ),
            ),
            cast_to=LookupTableRevisionDataUpdateResponse,
        )

    async def delete(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataDeleteResponse:
        """
        Delete the Lookup Table Revision Data for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataDeleteResponse,
        )

    async def archieve(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_type: Literal["application/jsonl", "text/csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataArchieveResponse:
        """
        Get a URL which you can use to download the data for the specified archived
        Lookup Table Revision:

        - The `contentType` request parameter is required.
        - The returned URL is presigned - you can copy it into a browser and the data
          file is downloaded locally.
        - The upload URL is time limited - the `expiry` time is given in the response
          and the URL is valid for **_one hour_**.

        Args:
          content_type: The content type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/archived",
            body=await async_maybe_transform(
                {"content_type": content_type},
                lookup_table_revision_data_archieve_params.LookupTableRevisionDataArchieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataArchieveResponse,
        )

    async def copy(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        revision_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataCopyResponse:
        """
        Copy the Lookup Table Revision Data from a source Revision to an optional target
        Revision:

        - If you omit a target `revisionId`, then the source Revision and its Data is
          duplicated. The new Revision is given the source Revision's name appended with
          "Copy" but is assigned a new unique id.
        - If you specify a target `revisionId` to copy the source Revision and its Data
          to, you must ensure that the target Revision has a Data schema that matches
          the source Revision's Data schema otherwise you'll receive an error

        Args:
          revision_id: The target Revision id that the source Revision's data will be copied to.
              _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/copy",
            body=await async_maybe_transform(
                {"revision_id": revision_id}, lookup_table_revision_data_copy_params.LookupTableRevisionDataCopyParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataCopyResponse,
        )

    async def delete_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataDeleteKeyResponse:
        """
        Delete a Lookup Table Revision Data entry by lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          version: The version of the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            body=await async_maybe_transform(
                {"version": version},
                lookup_table_revision_data_delete_key_params.LookupTableRevisionDataDeleteKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataDeleteKeyResponse,
        )

    async def generate_download_url(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_length: int,
        content_type: Literal["application/jsonl", "text/csv"],
        file_name: str,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataGenerateDownloadURLResponse:
        """
        Generate a URL which can be used to upload a data file for creating or updating
        the Lookup Table Revision's data:

        - An upload URL is returned together with an UPLOAD `jobId`.
        - You can then upload your data file using a PUT request using the returned
          upload URL as the endpoint. For the PUT request, map the headers returned and
          their values and in the request body select the specified CSV or JSONL file
          containing the Revision Data to upload.
        - You can use the returned UPLOAD `jobId` with the
          [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
          or the
          [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
          endpoints for any follow-up or troubleshooting.

        **Important:**

        - The `contentLength` request parameter is required.
        - The upload URL is time limited - it is valid for **_one minute_**.

        Args:
          content_length: The size of the file body in bytes. For example: `"contentLength": 485`, where
              485 is the size in bytes of the file to upload.

          content_type: The content type

          file_name: The name of the file to be uploaded.

          version: Version of the Lookup Table Revision Data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/generateuploadurl",
            body=await async_maybe_transform(
                {
                    "content_length": content_length,
                    "content_type": content_type,
                    "file_name": file_name,
                    "version": version,
                },
                lookup_table_revision_data_generate_download_url_params.LookupTableRevisionDataGenerateDownloadURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataGenerateDownloadURLResponse,
        )

    async def retrieve_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataRetrieveKeyResponse:
        """
        Retrieve a Lookup Table Revision Data item for the given lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataRetrieveKeyResponse,
        )

    async def update_key(
        self,
        lookup_key: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        item: Dict[str, object],
        additional: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataUpdateKeyResponse:
        """
        Create/update a Lookup Table Revision Data item by lookup key.

        **NOTES:**

        - To obtain the lookup key for a Revision's data items, use the
          [Get LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionData)
          endpoint in this section and use the `additional=lookupKey` query parameter.
        - If the Revision's Data schema uses multiple key fields, enter these as a
          comma-separated list for the `lookupKey` path parameter: .../key1,key2,key3
          and so on. Importantly, multiple keys must be _entered in the same order_ as
          they are configured in the Revision's Data schema.

        Args:
          item: The item you want to upsert

          additional: Comma separated list of additional fields. For example, you can use
              `additional=lookupKey` to get the lookup key returned for the Data item.

          version: The version of the LookupTableRevisionData.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not lookup_key:
            raise ValueError(f"Expected a non-empty value for `lookup_key` but received {lookup_key!r}")
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/{lookup_key}",
            body=await async_maybe_transform(
                {
                    "item": item,
                    "version": version,
                },
                lookup_table_revision_data_update_key_params.LookupTableRevisionDataUpdateKeyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"additional": additional},
                    lookup_table_revision_data_update_key_params.LookupTableRevisionDataUpdateKeyParams,
                ),
            ),
            cast_to=LookupTableRevisionDataUpdateKeyResponse,
        )


class LookupTableRevisionDataResourceWithRawResponse:
    def __init__(self, lookup_table_revision_data: LookupTableRevisionDataResource) -> None:
        self._lookup_table_revision_data = lookup_table_revision_data

        self.retrieve = to_raw_response_wrapper(
            lookup_table_revision_data.retrieve,
        )
        self.update = to_raw_response_wrapper(
            lookup_table_revision_data.update,
        )
        self.delete = to_raw_response_wrapper(
            lookup_table_revision_data.delete,
        )
        self.archieve = to_raw_response_wrapper(
            lookup_table_revision_data.archieve,
        )
        self.copy = to_raw_response_wrapper(
            lookup_table_revision_data.copy,
        )
        self.delete_key = to_raw_response_wrapper(
            lookup_table_revision_data.delete_key,
        )
        self.generate_download_url = to_raw_response_wrapper(
            lookup_table_revision_data.generate_download_url,
        )
        self.retrieve_key = to_raw_response_wrapper(
            lookup_table_revision_data.retrieve_key,
        )
        self.update_key = to_raw_response_wrapper(
            lookup_table_revision_data.update_key,
        )

    @cached_property
    def lookup_table_revision_data_jobs(self) -> LookupTableRevisionDataJobsResourceWithRawResponse:
        return LookupTableRevisionDataJobsResourceWithRawResponse(
            self._lookup_table_revision_data.lookup_table_revision_data_jobs
        )


class AsyncLookupTableRevisionDataResourceWithRawResponse:
    def __init__(self, lookup_table_revision_data: AsyncLookupTableRevisionDataResource) -> None:
        self._lookup_table_revision_data = lookup_table_revision_data

        self.retrieve = async_to_raw_response_wrapper(
            lookup_table_revision_data.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            lookup_table_revision_data.update,
        )
        self.delete = async_to_raw_response_wrapper(
            lookup_table_revision_data.delete,
        )
        self.archieve = async_to_raw_response_wrapper(
            lookup_table_revision_data.archieve,
        )
        self.copy = async_to_raw_response_wrapper(
            lookup_table_revision_data.copy,
        )
        self.delete_key = async_to_raw_response_wrapper(
            lookup_table_revision_data.delete_key,
        )
        self.generate_download_url = async_to_raw_response_wrapper(
            lookup_table_revision_data.generate_download_url,
        )
        self.retrieve_key = async_to_raw_response_wrapper(
            lookup_table_revision_data.retrieve_key,
        )
        self.update_key = async_to_raw_response_wrapper(
            lookup_table_revision_data.update_key,
        )

    @cached_property
    def lookup_table_revision_data_jobs(self) -> AsyncLookupTableRevisionDataJobsResourceWithRawResponse:
        return AsyncLookupTableRevisionDataJobsResourceWithRawResponse(
            self._lookup_table_revision_data.lookup_table_revision_data_jobs
        )


class LookupTableRevisionDataResourceWithStreamingResponse:
    def __init__(self, lookup_table_revision_data: LookupTableRevisionDataResource) -> None:
        self._lookup_table_revision_data = lookup_table_revision_data

        self.retrieve = to_streamed_response_wrapper(
            lookup_table_revision_data.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            lookup_table_revision_data.update,
        )
        self.delete = to_streamed_response_wrapper(
            lookup_table_revision_data.delete,
        )
        self.archieve = to_streamed_response_wrapper(
            lookup_table_revision_data.archieve,
        )
        self.copy = to_streamed_response_wrapper(
            lookup_table_revision_data.copy,
        )
        self.delete_key = to_streamed_response_wrapper(
            lookup_table_revision_data.delete_key,
        )
        self.generate_download_url = to_streamed_response_wrapper(
            lookup_table_revision_data.generate_download_url,
        )
        self.retrieve_key = to_streamed_response_wrapper(
            lookup_table_revision_data.retrieve_key,
        )
        self.update_key = to_streamed_response_wrapper(
            lookup_table_revision_data.update_key,
        )

    @cached_property
    def lookup_table_revision_data_jobs(self) -> LookupTableRevisionDataJobsResourceWithStreamingResponse:
        return LookupTableRevisionDataJobsResourceWithStreamingResponse(
            self._lookup_table_revision_data.lookup_table_revision_data_jobs
        )


class AsyncLookupTableRevisionDataResourceWithStreamingResponse:
    def __init__(self, lookup_table_revision_data: AsyncLookupTableRevisionDataResource) -> None:
        self._lookup_table_revision_data = lookup_table_revision_data

        self.retrieve = async_to_streamed_response_wrapper(
            lookup_table_revision_data.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            lookup_table_revision_data.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            lookup_table_revision_data.delete,
        )
        self.archieve = async_to_streamed_response_wrapper(
            lookup_table_revision_data.archieve,
        )
        self.copy = async_to_streamed_response_wrapper(
            lookup_table_revision_data.copy,
        )
        self.delete_key = async_to_streamed_response_wrapper(
            lookup_table_revision_data.delete_key,
        )
        self.generate_download_url = async_to_streamed_response_wrapper(
            lookup_table_revision_data.generate_download_url,
        )
        self.retrieve_key = async_to_streamed_response_wrapper(
            lookup_table_revision_data.retrieve_key,
        )
        self.update_key = async_to_streamed_response_wrapper(
            lookup_table_revision_data.update_key,
        )

    @cached_property
    def lookup_table_revision_data_jobs(self) -> AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse:
        return AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse(
            self._lookup_table_revision_data.lookup_table_revision_data_jobs
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["LookupTableListParams"]


class LookupTableListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    additional: SequenceNotStr[str]
    """
    Comma separated list of additional non-default fields to be included in the
    response. For example,if you want to include the active Revision for each of the
    Lookup Tables returned, set `additional=activeRevision` in the query.
    """

    codes: SequenceNotStr[str]
    """List of Lookup Table codes to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """Token to supply for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Lookup Tables to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataDeleteKeyParams"]


class LookupTableRevisionDataDeleteKeyParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    lookup_table_revision_id: Required[Annotated[str, PropertyInfo(alias="lookupTableRevisionId")]]

    version: int
    """The version of the Lookup Table Revision Data."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.lookup_tables.lookup_table_revision_data import (
    lookup_table_revision_data_job_list_params,
    lookup_table_revision_data_job_download_params,
)
from ....types.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_job_list_response import (
    LookupTableRevisionDataJobListResponse,
)
from ....types.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_job_delete_response import (
    LookupTableRevisionDataJobDeleteResponse,
)
from ....types.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_job_download_response import (
    LookupTableRevisionDataJobDownloadResponse,
)
from ....types.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_job_retrieve_response import (
    LookupTableRevisionDataJobRetrieveResponse,
)

__all__ = ["LookupTableRevisionDataJobsResource", "AsyncLookupTableRevisionDataJobsResource"]


class LookupTableRevisionDataJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LookupTableRevisionDataJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LookupTableRevisionDataJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LookupTableRevisionDataJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LookupTableRevisionDataJobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobRetrieveResponse:
        """
        Get the Lookup Table Revision Data job Response for given job id.

        **NOTE:** Use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint to list the Data job Responses for a specific Revision.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobRetrieveResponse,
        )

    def list(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[LookupTableRevisionDataJobListResponse]:
        """
        List the Lookup Table Revision Data job Responses for the given Lookup Table
        Revision.

        There are four types of Revision Data jobs:

        - **COPY**. Job runs when you use the
          [Copy LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/CopyLookupTableRevisionData)
          endpoint which returns the `jobId`.
        - **UPLOAD**. Job runs when you use the
          [Generate LookupTableRevisionData Upload URL](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GenerateLookupTableDataUploadUrl)
          endpoint which returns the `jobId`.
        - **DOWNLOAD**. Job runs when you use the
          [](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/TriggerLookupTableRevisionDataDownloadJob)
          endpoint which returns the `jobId`.
        - **ARCHIVE**. Job runs when you either manually change a DRAFT Revision to
          PUBLISHED using the
          [Update LookupTableRevision Status](https://www.m3ter.com/docs/api#tag/LookupTableRevision/operation/UpdateLookupTableRevisionStatus)
          endpoint or you publish a DRAFT Revision and the existing PUBLISHED Revision
          is archived.

        **NOTE:** This endpoint returns the id of each Data job. You then use:

        - The
          [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
          endpoint to retrieve a specific Data job Response.
        - The
          [Delete LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/DeleteLookupTableRevisionDataJobResponse)
          to delete a specific Data job Response.

        Args:
          next_token: The nextToken for multi page retrievals

          page_size: The number of Lookup Table Revision Data Job Responses to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs",
            page=SyncCursor[LookupTableRevisionDataJobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_revision_data_job_list_params.LookupTableRevisionDataJobListParams,
                ),
            ),
            model=LookupTableRevisionDataJobListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobDeleteResponse:
        """
        Delete the LookupTableRevisionData Job Response for given job id.

        **NOTE:** Use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint to list the Data job Responses for a specific Revision.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobDeleteResponse,
        )

    def download(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_type: Literal["application/jsonl", "text/csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobDownloadResponse:
        """Trigger an URL job to download the Lookup Table Revision Data.

        The URL download
        Data `jobId` is returned and you can then use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint or the
        [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
        endpoint to retrieve the URL and perform the Revision data Download.

        Args:
          content_type: The content type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/download",
            body=maybe_transform(
                {"content_type": content_type},
                lookup_table_revision_data_job_download_params.LookupTableRevisionDataJobDownloadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobDownloadResponse,
        )


class AsyncLookupTableRevisionDataJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLookupTableRevisionDataJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLookupTableRevisionDataJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobRetrieveResponse:
        """
        Get the Lookup Table Revision Data job Response for given job id.

        **NOTE:** Use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint to list the Data job Responses for a specific Revision.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobRetrieveResponse,
        )

    def list(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LookupTableRevisionDataJobListResponse, AsyncCursor[LookupTableRevisionDataJobListResponse]]:
        """
        List the Lookup Table Revision Data job Responses for the given Lookup Table
        Revision.

        There are four types of Revision Data jobs:

        - **COPY**. Job runs when you use the
          [Copy LookupTableRevisionData](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/CopyLookupTableRevisionData)
          endpoint which returns the `jobId`.
        - **UPLOAD**. Job runs when you use the
          [Generate LookupTableRevisionData Upload URL](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GenerateLookupTableDataUploadUrl)
          endpoint which returns the `jobId`.
        - **DOWNLOAD**. Job runs when you use the
          [](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/TriggerLookupTableRevisionDataDownloadJob)
          endpoint which returns the `jobId`.
        - **ARCHIVE**. Job runs when you either manually change a DRAFT Revision to
          PUBLISHED using the
          [Update LookupTableRevision Status](https://www.m3ter.com/docs/api#tag/LookupTableRevision/operation/UpdateLookupTableRevisionStatus)
          endpoint or you publish a DRAFT Revision and the existing PUBLISHED Revision
          is archived.

        **NOTE:** This endpoint returns the id of each Data job. You then use:

        - The
          [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
          endpoint to retrieve a specific Data job Response.
        - The
          [Delete LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/DeleteLookupTableRevisionDataJobResponse)
          to delete a specific Data job Response.

        Args:
          next_token: The nextToken for multi page retrievals

          page_size: The number of Lookup Table Revision Data Job Responses to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs",
            page=AsyncCursor[LookupTableRevisionDataJobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_revision_data_job_list_params.LookupTableRevisionDataJobListParams,
                ),
            ),
            model=LookupTableRevisionDataJobListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        lookup_table_revision_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobDeleteResponse:
        """
        Delete the LookupTableRevisionData Job Response for given job id.

        **NOTE:** Use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint to list the Data job Responses for a specific Revision.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobDeleteResponse,
        )

    async def download(
        self,
        lookup_table_revision_id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        content_type: Literal["application/jsonl", "text/csv"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionDataJobDownloadResponse:
        """Trigger an URL job to download the Lookup Table Revision Data.

        The URL download
        Data `jobId` is returned and you can then use the
        [List LookupTableRevisionData Jobs](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/ListLookupTableRevisionDataJobs)
        endpoint or the
        [Get LookupTableRevisionData Job Response](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/GetLookupTableRevisionDataJobResponse)
        endpoint to retrieve the URL and perform the Revision data Download.

        Args:
          content_type: The content type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not lookup_table_revision_id:
            raise ValueError(
                f"Expected a non-empty value for `lookup_table_revision_id` but received {lookup_table_revision_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{lookup_table_revision_id}/data/jobs/download",
            body=await async_maybe_transform(
                {"content_type": content_type},
                lookup_table_revision_data_job_download_params.LookupTableRevisionDataJobDownloadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionDataJobDownloadResponse,
        )


class LookupTableRevisionDataJobsResourceWithRawResponse:
    def __init__(self, lookup_table_revision_data_jobs: LookupTableRevisionDataJobsResource) -> None:
        self._lookup_table_revision_data_jobs = lookup_table_revision_data_jobs

        self.retrieve = to_raw_response_wrapper(
            lookup_table_revision_data_jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            lookup_table_revision_data_jobs.list,
        )
        self.delete = to_raw_response_wrapper(
            lookup_table_revision_data_jobs.delete,
        )
        self.download = to_raw_response_wrapper(
            lookup_table_revision_data_jobs.download,
        )


class AsyncLookupTableRevisionDataJobsResourceWithRawResponse:
    def __init__(self, lookup_table_revision_data_jobs: AsyncLookupTableRevisionDataJobsResource) -> None:
        self._lookup_table_revision_data_jobs = lookup_table_revision_data_jobs

        self.retrieve = async_to_raw_response_wrapper(
            lookup_table_revision_data_jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            lookup_table_revision_data_jobs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lookup_table_revision_data_jobs.delete,
        )
        self.download = async_to_raw_response_wrapper(
            lookup_table_revision_data_jobs.download,
        )


class LookupTableRevisionDataJobsResourceWithStreamingResponse:
    def __init__(self, lookup_table_revision_data_jobs: LookupTableRevisionDataJobsResource) -> None:
        self._lookup_table_revision_data_jobs = lookup_table_revision_data_jobs

        self.retrieve = to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.list,
        )
        self.delete = to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.delete,
        )
        self.download = to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.download,
        )


class AsyncLookupTableRevisionDataJobsResourceWithStreamingResponse:
    def __init__(self, lookup_table_revision_data_jobs: AsyncLookupTableRevisionDataJobsResource) -> None:
        self._lookup_table_revision_data_jobs = lookup_table_revision_data_jobs

        self.retrieve = async_to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.delete,
        )
        self.download = async_to_streamed_response_wrapper(
            lookup_table_revision_data_jobs.download,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import credit_reason_list_params, credit_reason_create_params, credit_reason_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.credit_reason_response import CreditReasonResponse

__all__ = ["CreditReasonsResource", "AsyncCreditReasonsResource"]


class CreditReasonsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CreditReasonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CreditReasonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditReasonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CreditReasonsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """Create a new Credit Reason for your Organization.

        When you've created a Credit
        Reason, it becomes available as a credit type for adding Credit line items to
        Bills. See [Credits](https://www.m3ter.com/docs/api#tag/Credits).

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/picklists/creditreasons",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                credit_reason_create_params.CreditReasonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Retrieve the Credit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Update the Credit Reason with the given UUID.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                credit_reason_update_params.CreditReasonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[CreditReasonResponse]:
        """Retrieve a list of the Credit Reason entities created for your Organization.

        You
        can filter the list returned for the call by Credit Reason ID, Credit Reason
        short code, or by Archive status.

        Args:
          archived: TRUE / FALSE archived flag to filter the list. CreditReasons can be archived
              once they are obsolete.

              - TRUE includes archived CreditReasons.
              - FALSE excludes CreditReasons that are archived.

          codes: List of Credit Reason short codes to retrieve.

          ids: List of Credit Reason IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of credit reasons to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/creditreasons",
            page=SyncCursor[CreditReasonResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    credit_reason_list_params.CreditReasonListParams,
                ),
            ),
            model=CreditReasonResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Delete the Credit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )


class AsyncCreditReasonsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCreditReasonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditReasonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditReasonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCreditReasonsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """Create a new Credit Reason for your Organization.

        When you've created a Credit
        Reason, it becomes available as a credit type for adding Credit line items to
        Bills. See [Credits](https://www.m3ter.com/docs/api#tag/Credits).

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/picklists/creditreasons",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                credit_reason_create_params.CreditReasonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Retrieve the Credit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | Omit = omit,
        code: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Update the Credit Reason with the given UUID.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                credit_reason_update_params.CreditReasonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CreditReasonResponse, AsyncCursor[CreditReasonResponse]]:
        """Retrieve a list of the Credit Reason entities created for your Organization.

        You
        can filter the list returned for the call by Credit Reason ID, Credit Reason
        short code, or by Archive status.

        Args:
          archived: TRUE / FALSE archived flag to filter the list. CreditReasons can be archived
              once they are obsolete.

              - TRUE includes archived CreditReasons.
              - FALSE excludes CreditReasons that are archived.

          codes: List of Credit Reason short codes to retrieve.

          ids: List of Credit Reason IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of credit reasons to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/creditreasons",
            page=AsyncCursor[CreditReasonResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    credit_reason_list_params.CreditReasonListParams,
                ),
            ),
            model=CreditReasonResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditReasonResponse:
        """
        Delete the Credit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/picklists/creditreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditReasonResponse,
        )


class CreditReasonsResourceWithRawResponse:
    def __init__(self, credit_reasons: CreditReasonsResource) -> None:
        self._credit_reasons = credit_reasons

        self.create = to_raw_response_wrapper(
            credit_reasons.create,
        )
        self.retrieve = to_raw_response_wrapper(
            credit_reasons.retrieve,
        )
        self.update = to_raw_response_wrapper(
            credit_reasons.update,
        )
        self.list = to_raw_response_wrapper(
            credit_reasons.list,
        )
        self.delete = to_raw_response_wrapper(
            credit_reasons.delete,
        )


class AsyncCreditReasonsResourceWithRawResponse:
    def __init__(self, credit_reasons: AsyncCreditReasonsResource) -> None:
        self._credit_reasons = credit_reasons

        self.create = async_to_raw_response_wrapper(
            credit_reasons.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            credit_reasons.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            credit_reasons.update,
        )
        self.list = async_to_raw_response_wrapper(
            credit_reasons.list,
        )
        self.delete = async_to_raw_response_wrapper(
            credit_reasons.delete,
        )


class CreditReasonsResourceWithStreamingResponse:
    def __init__(self, credit_reasons: CreditReasonsResource) -> None:
        self._credit_reasons = credit_reasons

        self.create = to_streamed_response_wrapper(
            credit_reasons.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            credit_reasons.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            credit_reasons.update,
        )
        self.list = to_streamed_response_wrapper(
            credit_reasons.list,
        )
        self.delete = to_streamed_response_wrapper(
            credit_reasons.delete,
        )


class AsyncCreditReasonsResourceWithStreamingResponse:
    def __init__(self, credit_reasons: AsyncCreditReasonsResource) -> None:
        self._credit_reasons = credit_reasons

        self.create = async_to_streamed_response_wrapper(
            credit_reasons.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            credit_reasons.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            credit_reasons.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credit_reasons.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            credit_reasons.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CreditLineItemResponse"]


class CreditLineItemResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    amount: Optional[float] = None
    """The credit amount."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this line item."""

    credit_reason_id: Optional[str] = FieldInfo(alias="creditReasonId", default=None)
    """The UUID of the credit reason for this credit line item."""

    description: Optional[str] = None

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the line item was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the line item was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this line item."""

    line_item_type: Optional[
        Literal[
            "STANDING_CHARGE",
            "USAGE",
            "COUNTER_RUNNING_TOTAL_CHARGE",
            "COUNTER_ADJUSTMENT_DEBIT",
            "COUNTER_ADJUSTMENT_CREDIT",
            "USAGE_CREDIT",
            "MINIMUM_SPEND",
            "MINIMUM_SPEND_REFUND",
            "CREDIT_DEDUCTION",
            "MANUAL_ADJUSTMENT",
            "CREDIT_MEMO",
            "DEBIT_MEMO",
            "COMMITMENT_CONSUMED",
            "COMMITMENT_FEE",
            "OVERAGE_SURCHARGE",
            "OVERAGE_USAGE",
            "BALANCE_CONSUMED",
            "BALANCE_FEE",
            "AD_HOC",
        ]
    ] = FieldInfo(alias="lineItemType", default=None)

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)

    referenced_bill_id: Optional[str] = FieldInfo(alias="referencedBillId", default=None)

    referenced_line_item_id: Optional[str] = FieldInfo(alias="referencedLineItemId", default=None)

    service_period_end_date: Optional[datetime] = FieldInfo(alias="servicePeriodEndDate", default=None)

    service_period_start_date: Optional[datetime] = FieldInfo(alias="servicePeriodStartDate", default=None)

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .statements import (
    StatementsResource,
    AsyncStatementsResource,
    StatementsResourceWithRawResponse,
    AsyncStatementsResourceWithRawResponse,
    StatementsResourceWithStreamingResponse,
    AsyncStatementsResourceWithStreamingResponse,
)
from .statement_jobs import (
    StatementJobsResource,
    AsyncStatementJobsResource,
    StatementJobsResourceWithRawResponse,
    AsyncStatementJobsResourceWithRawResponse,
    StatementJobsResourceWithStreamingResponse,
    AsyncStatementJobsResourceWithStreamingResponse,
)
from .statement_definitions import (
    StatementDefinitionsResource,
    AsyncStatementDefinitionsResource,
    StatementDefinitionsResourceWithRawResponse,
    AsyncStatementDefinitionsResourceWithRawResponse,
    StatementDefinitionsResourceWithStreamingResponse,
    AsyncStatementDefinitionsResourceWithStreamingResponse,
)

__all__ = [
    "StatementJobsResource",
    "AsyncStatementJobsResource",
    "StatementJobsResourceWithRawResponse",
    "AsyncStatementJobsResourceWithRawResponse",
    "StatementJobsResourceWithStreamingResponse",
    "AsyncStatementJobsResourceWithStreamingResponse",
    "StatementDefinitionsResource",
    "AsyncStatementDefinitionsResource",
    "StatementDefinitionsResourceWithRawResponse",
    "AsyncStatementDefinitionsResourceWithRawResponse",
    "StatementDefinitionsResourceWithStreamingResponse",
    "AsyncStatementDefinitionsResourceWithStreamingResponse",
    "StatementsResource",
    "AsyncStatementsResource",
    "StatementsResourceWithRawResponse",
    "AsyncStatementsResourceWithRawResponse",
    "StatementsResourceWithStreamingResponse",
    "AsyncStatementsResourceWithStreamingResponse",
]
